@@ -1,9 +1,9 @@
 /*
- * "$Id: snmp.c 9838 2011-06-19 04:28:25Z mike $"
+ * "$Id: snmp.c 10535 2012-06-22 03:45:53Z mike $"
  *
  *   SNMP discovery backend for CUPS.
  *
- *   Copyright 2007-2011 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 2006-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -251,7 +251,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
 #ifdef AF_INET6
   if ((ipv6 = _cupsSNMPOpen(AF_INET6)) < 0)
-    return (1);
+    perror("DEBUG: Unable to create IPv6 socket");
 #else
   ipv6 = -1;
 #endif /* AF_INET6 */
@@ -1112,12 +1112,18 @@ read_snmp_response(int fd)		/* I - SNMP socket file descriptor */
 
     case DEVICE_URI :
 	if (device && packet.object_type == CUPS_ASN1_OCTET_STRING &&
-	    !device->uri && packet.object_value.string.num_bytes > 0)
+	    !device->uri && packet.object_value.string.num_bytes > 3)
 	{
 	 /*
 	  * Update an existing cache entry...
 	  */
 
+          char	scheme[32],		/* URI scheme */
+		userpass[256],		/* Username:password in URI */
+		hostname[256],		/* Hostname in URI */
+		resource[1024];		/* Resource path in URI */
+	  int	port;			/* Port number in URI */
+
 	  if (!strncmp((char *)packet.object_value.string.bytes, "lpr:", 4))
 	  {
 	   /*
@@ -1127,7 +1133,13 @@ read_snmp_response(int fd)		/* I - SNMP socket file descriptor */
 	    packet.object_value.string.bytes[2] = 'd';
 	  }
 
-	  device->uri = strdup((char *)packet.object_value.string.bytes);
+          if (httpSeparateURI(HTTP_URI_CODING_ALL,
+                              (char *)packet.object_value.string.bytes,
+                              scheme, sizeof(scheme),
+                              userpass, sizeof(userpass),
+                              hostname, sizeof(hostname), &port,
+                              resource, sizeof(resource)) >= HTTP_URI_OK)
+	    device->uri = strdup((char *)packet.object_value.string.bytes);
 	}
 	break;
   }
@@ -1371,5 +1383,5 @@ update_cache(snmp_cache_t *device,	/* I - Device */
 
 
 /*
- * End of "$Id: snmp.c 9838 2011-06-19 04:28:25Z mike $".
+ * End of "$Id: snmp.c 10535 2012-06-22 03:45:53Z mike $".
  */
