@@ -1,9 +1,9 @@
 /*
- * "$Id: ipp-var.c 9181 2010-06-22 21:42:44Z mike $"
+ * "$Id: ipp-var.c 10431 2012-04-23 19:19:19Z mike $"
  *
- *   CGI <-> IPP variable routines for the Common UNIX Printing System (CUPS).
+ *   CGI <-> IPP variable routines for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -362,7 +362,7 @@ cgiMoveJobs(http_t     *http,		/* I - Connection to server */
 	*/
 
         cgiStartHTML(cgiText(_("Move Job")));
-	cgiShowIPPError(_("Unable to find destination for job!"));
+	cgiShowIPPError(_("Unable to find destination for job"));
 	cgiEndHTML();
 	return;
       }
@@ -404,7 +404,7 @@ cgiMoveJobs(http_t     *http,		/* I - Connection to server */
         * If the name is not the same as the current destination, add it!
 	*/
 
-        if (strcasecmp(name, dest))
+        if (_cups_strcasecmp(name, dest))
 	{
 	  cgiSetArray("JOB_PRINTER_URI", i, attr->values[0].string.text);
 	  cgiSetArray("JOB_PRINTER_NAME", i, name);
@@ -595,7 +595,7 @@ cgiPrintCommand(http_t     *http,	/* I - Connection to server */
   if ((job_id = cupsCreateJob(http, dest, title,
 			      1, &hold_option)) < 1)
   {
-    cgiSetVariable("MESSAGE", cgiText(_("Unable to send command to printer driver!")));
+    cgiSetVariable("MESSAGE", cgiText(_("Unable to send command to printer driver")));
     cgiSetVariable("ERROR", cupsLastErrorString());
     cgiStartHTML(title);
     cgiCopyTemplateLang("error.tmpl");
@@ -615,7 +615,7 @@ cgiPrintCommand(http_t     *http,	/* I - Connection to server */
 
   if (cupsLastError() >= IPP_REDIRECTION_OTHER_SITE)
   {
-    cgiSetVariable("MESSAGE", cgiText(_("Unable to send command to printer driver!")));
+    cgiSetVariable("MESSAGE", cgiText(_("Unable to send command to printer driver")));
     cgiSetVariable("ERROR", cupsLastErrorString());
     cgiStartHTML(title);
     cgiCopyTemplateLang("error.tmpl");
@@ -792,7 +792,7 @@ cgiPrintTestPage(http_t     *http,	/* I - Connection to server */
   cgiStartHTML(cgiText(_("Print Test Page")));
 
   if (cupsLastError() > IPP_OK_CONFLICT)
-    cgiShowIPPError(_("Unable to print test page:"));
+    cgiShowIPPError(_("Unable to print test page"));
   else
   {
     cgiSetVariable("PRINTER_NAME", dest);
@@ -906,12 +906,12 @@ cgiRewriteURL(const char *uri,		/* I - Current URI */
     * Map local access to a local URI...
     */
 
-    if (!strcasecmp(hostname, "127.0.0.1") ||
-	!strcasecmp(hostname, "[::1]") ||
-	!strcasecmp(hostname, "localhost") ||
-	!strncasecmp(hostname, "localhost.", 10) ||
-	!strcasecmp(hostname, server) ||
-	!strcasecmp(hostname, servername))
+    if (!_cups_strcasecmp(hostname, "127.0.0.1") ||
+	!_cups_strcasecmp(hostname, "[::1]") ||
+	!_cups_strcasecmp(hostname, "localhost") ||
+	!_cups_strncasecmp(hostname, "localhost.", 10) ||
+	!_cups_strcasecmp(hostname, server) ||
+	!_cups_strcasecmp(hostname, servername))
     {
      /*
       * Make URI relative to the current server...
@@ -1219,7 +1219,7 @@ cgiSetIPPObjectVars(
 	             "%dx%d%s", attr->values[i].resolution.xres,
 		     attr->values[i].resolution.yres,
 		     attr->values[i].resolution.units == IPP_RES_PER_INCH ?
-			 "dpi" : "dpc");
+			 "dpi" : "dpcm");
 	    break;
 
 	case IPP_TAG_URI :
@@ -1347,7 +1347,7 @@ cgiSetIPPVars(ipp_t      *response,	/* I - Response data to be copied... */
 	     (filter->value_tag >= IPP_TAG_TEXTLANG &&
 	      filter->value_tag <= IPP_TAG_MIMETYPE)) &&
 	    filter->values[0].string.text != NULL &&
-	    !strcasecmp(filter->values[0].string.text, filter_value))
+	    !_cups_strcasecmp(filter->values[0].string.text, filter_value))
 	  break;
 
       if (!filter)
@@ -1481,9 +1481,9 @@ cgiShowJobs(http_t     *http,		/* I - Connection to server */
       first = 0;
 
     if ((var = cgiGetVariable("ORDER")) != NULL)
-      ascending = !strcasecmp(var, "asc");
+      ascending = !_cups_strcasecmp(var, "asc");
     else
-      ascending = !which_jobs || !strcasecmp(which_jobs, "not-completed");
+      ascending = !which_jobs || !_cups_strcasecmp(which_jobs, "not-completed");
 
     section = cgiGetVariable("SECTION");
 
@@ -1588,5 +1588,5 @@ cgiText(const char *message)		/* I - Message */
 
 
 /*
- * End of "$Id: ipp-var.c 9181 2010-06-22 21:42:44Z mike $".
+ * End of "$Id: ipp-var.c 10431 2012-04-23 19:19:19Z mike $".
  */
