@@ -1,9 +1,9 @@
 /*
- * "$Id: client.h 8685 2009-05-26 22:01:23Z mike $"
+ * "$Id: client.h 10374 2012-03-22 20:30:20Z mike $"
  *
- *   Client definitions for the Common UNIX Printing System (CUPS) scheduler.
+ *   Client definitions for the CUPS scheduler.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -32,8 +32,10 @@ struct cupsd_client_s
   http_state_t		operation;	/* Request operation */
   off_t			bytes;		/* Bytes transferred for this request */
   int			type;		/* AuthType for username */
-  char			username[256],	/* Username from Authorization: line */
-			password[33],	/* Password from Authorization: line */
+  char			username[HTTP_MAX_VALUE],
+					/* Username from Authorization: line */
+			password[HTTP_MAX_VALUE],
+					/* Password from Authorization: line */
 			uri[HTTP_MAX_URI],
 					/* Localized URL/URI for GET/PUT */
 			*filename,	/* Filename of output file */
@@ -56,10 +58,7 @@ struct cupsd_client_s
   int			serverport;	/* Server port for connection */
 #ifdef HAVE_GSSAPI
   int			have_gss;	/* Have GSS credentials? */
-  gss_cred_id_t 	gss_creds;	/* Delegated credentials from client */
-  unsigned		gss_flags;	/* Credential flags */
-  gss_buffer_desc 	gss_output_token;
-					/* Output token for Negotiate header */
+  uid_t			gss_uid;	/* User ID for local prints */
 #endif /* HAVE_GSSAPI */
 #ifdef HAVE_AUTHORIZATION_H
   AuthorizationRef	authref;	/* Authorization ref */
@@ -134,7 +133,12 @@ extern void	cupsdStopListening(void);
 extern void	cupsdUpdateCGI(void);
 extern void	cupsdWriteClient(cupsd_client_t *con);
 
+#ifdef HAVE_SSL
+extern int	cupsdEndTLS(cupsd_client_t *con);
+extern int	cupsdStartTLS(cupsd_client_t *con);
+#endif /* HAVE_SSL */
+
 
 /*
- * End of "$Id: client.h 8685 2009-05-26 22:01:23Z mike $".
+ * End of "$Id: client.h 10374 2012-03-22 20:30:20Z mike $".
  */
