@@ -1,9 +1,9 @@
 /*
- * "$Id: server.c 9104 2010-04-12 04:03:53Z mike $"
+ * "$Id: server.c 10490 2012-05-21 17:40:22Z mike $"
  *
  *   Server start/stop routines for the CUPS scheduler.
  *
- *   Copyright 2007-2010 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -34,7 +34,7 @@
  * Local globals...
  */
 
-static int	started = 0;
+static int		started = 0;	/* Did we start the server already? */
 
 
 /*
@@ -44,6 +44,12 @@ static int	started = 0;
 void
 cupsdStartServer(void)
 {
+ /*
+  * Start color management (as needed)...
+  */
+
+  cupsdStartColor();
+
  /*
   * Create the default security profile...
   */
@@ -56,7 +62,6 @@ cupsdStartServer(void)
 
   cupsdStartListening();
   cupsdStartBrowsing();
-  cupsdStartPolling();
 
  /*
   * Create a pipe for CGI processes...
@@ -95,12 +100,17 @@ cupsdStopServer(void)
     return;
 
  /*
-  * Close all network clients and stop all jobs...
+  * Stop color management (as needed)...
+  */
+
+  cupsdStopColor();
+
+ /*
+  * Close all network clients...
   */
 
   cupsdCloseAllClients();
   cupsdStopListening();
-  cupsdStopPolling();
   cupsdStopBrowsing();
   cupsdStopAllNotifiers();
   cupsdDeleteAllCerts();
@@ -126,16 +136,6 @@ cupsdStopServer(void)
     CGIPipes[1] = -1;
   }
 
-#ifdef HAVE_NOTIFY_POST
- /*
-  * Send one last notification as the server shuts down.
-  */
-
-  cupsdLogMessage(CUPSD_LOG_DEBUG,
-                  "notify_post(\"com.apple.printerListChange\") last");
-  notify_post("com.apple.printerListChange");
-#endif /* HAVE_NOTIFY_POST */
-
  /*
   * Close all log files...
   */
@@ -180,5 +180,5 @@ cupsdStopServer(void)
 
 
 /*
- * End of "$Id: server.c 9104 2010-04-12 04:03:53Z mike $".
+ * End of "$Id: server.c 10490 2012-05-21 17:40:22Z mike $".
  */
